@@ -117,7 +117,7 @@ License:
 
 PuDB is licensed to you under the MIT/X Consortium license:
 
-Copyright (c) 2009-13 Andreas Kloeckner and contributors
+Copyright (c) 2009-16 Andreas Kloeckner and contributors
 
 Permission is hereby granted, free of charge, to any person
 obtaining a copy of this software and associated documentation
@@ -543,9 +543,12 @@ class FileSourceCodeProvider(SourceCodeProvider):
         from pudb.source_view import SourceLine, format_source
 
         if self.file_name == "<string>":
-            return [SourceLine(self, self.file_name)]
+            return [SourceLine(debugger_ui, self.file_name)]
 
         breakpoints = debugger_ui.debugger.get_file_breaks(self.file_name)[:]
+        breakpoints = [lineno for lineno in breakpoints if
+            any(bp.enabled
+                for bp in debugger_ui.debugger.get_breaks(self.file_name, lineno))]
         breakpoints += [i for f, i in debugger_ui.debugger.set_traces if f
             == self.file_name and debugger_ui.debugger.set_traces[f, i]]
         try:
@@ -568,7 +571,7 @@ class FileSourceCodeProvider(SourceCodeProvider):
             debugger_ui.message("Could not load source file '%s':\n\n%s" % (
                 self.file_name, "".join(format_exception(sys.exc_info()))),
                 title="Source Code Load Error")
-            return [SourceLine(self,
+            return [SourceLine(debugger_ui,
                 "Error while loading '%s'." % self.file_name)]
 
 
@@ -984,6 +987,9 @@ class DebuggerUI(FrameVarInfoKeeper):
             bp = self._get_bp_list()[pos]
             bp.enabled = not bp.enabled
 
+            sline = self.source[bp.line-1]
+            sline.set_breakpoint(bp.enabled)
+
             self.update_breakpoints()
 
         def examine_breakpoint(w, size, key):
@@ -1205,8 +1211,17 @@ class DebuggerUI(FrameVarInfoKeeper):
                 existing_breaks = self.debugger.get_breaks(
                         bp_source_identifier, lineno)
                 if existing_breaks:
-                    err = self.debugger.clear_break(bp_source_identifier, lineno)
-                    sline.set_breakpoint(False)
+                    err = None
+                    for bp in existing_breaks:
+                        if not bp.enabled:
+                            bp.enable()
+                            sline.set_breakpoint(True)
+                            # Unsure about this. Are multiple breakpoints even
+                            # possible?
+                            break
+                    else:
+                        err = self.debugger.clear_break(bp_source_identifier, lineno)
+                        sline.set_breakpoint(False)
                 else:
                     file_lineno = (bp_source_identifier, lineno)
                     if file_lineno in self.debugger.set_traces:
@@ -1278,7 +1293,9 @@ class DebuggerUI(FrameVarInfoKeeper):
 
             def build_filtered_mod_list(filt_string=""):
                 modules = sorted(name
-                        for name, mod in sys.modules.items()
+                        # mod_exists may change the size of sys.modules,
+                        # causing this to crash. Copy to a list.
+                        for name, mod in list(sys.modules.items())
                         if mod_exists(mod))
 
                 result = [urwid.AttrMap(SelectableText(mod),
@@ -1345,7 +1362,7 @@ class DebuggerUI(FrameVarInfoKeeper):
                                     format_exception(sys.exc_info()))),
                                 title="Import Error")
                         else:
-                            show_mod(sys.modules[str(new_mod_name)])
+                            show_mod(__import__(str(new_mod_name)))
                             break
                     else:
                         show_mod(sys.modules[widget.base_widget.get_text()[0]])
@@ -1357,7 +1374,12 @@ class DebuggerUI(FrameVarInfoKeeper):
                     if widget is not new_mod_entry:
                         mod_name = widget.base_widget.get_text()[0]
                         mod = sys.modules[mod_name]
-                        reload(mod)
+                        if PY3:
+                            reload(mod)  # noqa (undef on Py3)
+                        else:
+                            import importlib
+                            importlib.reload(mod)
+
                         self.message("'%s' was successfully reloaded." % mod_name)
 
                         if self.source_code_provider is not None:
@@ -2073,7 +2095,7 @@ class DebuggerUI(FrameVarInfoKeeper):
                 self.message("Package 'pygments' not found. "
                         "Syntax highlighting disabled.")
 
-        WELCOME_LEVEL = "e026"  # noqa
+        WELCOME_LEVEL = "e027"  # noqa
         if CONFIG["seen_welcome"] < WELCOME_LEVEL:
             CONFIG["seen_welcome"] = WELCOME_LEVEL
             from pudb import VERSION
@@ -2090,6 +2112,13 @@ class DebuggerUI(FrameVarInfoKeeper):
                     "(invoked by hitting '?' after this message) should get you "
                     "on your way.\n"
 
+                    "\nChanges in version 2016.2:\n\n"
+                    "- UI improvements for disabled breakpoints.\n"
+                    "- Bug fixes.\n"
+
+                    "\nChanges in version 2016.1:\n\n"
+                    "- Fix module browser on Py3.\n"
+
                     "\nChanges in version 2015.4:\n\n"
                     "- Support for (somewhat rudimentary) remote debugging\n"
                     "  through a telnet connection.\n"
@@ -2328,7 +2357,7 @@ class DebuggerUI(FrameVarInfoKeeper):
     def update_breakpoints(self):
         self.bp_walker[:] = [
                 BreakpointFrame(self.debugger.current_bp == (bp.file, bp.line),
-                    self._format_fname(bp.file), bp.line)
+                    self._format_fname(bp.file), bp)
                 for bp in self._get_bp_list()]
 
     def update_stack(self):
