@@ -1,4 +1,4 @@
-THEMES = ["classic", "vim", "dark vim", "midnight"]
+THEMES = ["classic", "vim", "dark vim", "midnight", "solarized", "agr-256"]
 
 from pudb.py3compat import execfile, raw_input
 import urwid
@@ -61,9 +61,13 @@ def get_palette(may_use_fancy_formats, theme="classic"):
         # {{{ source
 
         "breakpoint": ("black", "dark cyan"),
+        "disabled breakpoint": ("dark gray", "dark cyan"),
         "focused breakpoint": ("black", "dark green"),
+        "focused disabled breakpoint": ("dark gray", "dark green"),
         "current breakpoint": (add_setting("white", "bold"), "dark cyan"),
+        "disabled current breakpoint": (add_setting("dark gray", "bold"), "dark cyan"),
         "focused current breakpoint": (add_setting("white", "bold"), "dark green", "bold"),
+        "focused disabled current breakpoint": (add_setting("dark gray", "bold"), "dark green", "bold"),
 
         "source": (add_setting("yellow", "bold"), "dark blue"),
         "focused source": ("black", "dark green"),
@@ -247,10 +251,15 @@ def get_palette(may_use_fancy_formats, theme="classic"):
             # {{{ breakpoint view
 
             "breakpoint": ("light gray", "dark gray"),
+            "disabled breakpoint": ("black", "dark gray"),
             "focused breakpoint": ("light gray", "light blue"),
+            "focused disabled breakpoint": ("black", "light blue"),
             "current breakpoint": (add_setting("white", "bold"), "dark gray"),
+            "disabled current breakpoint": ("black", "dark gray"),
             "focused current breakpoint":
                 (add_setting("white", "bold"), "light blue"),
+            "focused disabled current breakpoint":
+                ("black", "light blue"),
 
             # }}}
 
@@ -372,6 +381,15 @@ def get_palette(may_use_fancy_formats, theme="classic"):
 
             "search box": ("default", "default"),
 
+            "breakpoint": ("white", "default"),
+            "disabled breakpoint": ("dark gray", "default"),
+            "focused breakpoint": ("black", "dark green"),
+            "focused disabled breakpoint": ("dark gray", "dark green"),
+            "current breakpoint": (add_setting("white", "bold"), "default"),
+            "disabled current breakpoint": (add_setting("dark gray", "bold"), "default"),
+            "focused current breakpoint": (add_setting("white", "bold"), "dark green", "bold"),
+            "focused disabled current breakpoint": (add_setting("dark gray", "bold"), "dark green", "bold"),
+
             "source": ("white", "default"),
             "highlighted source": ("white", "light cyan"),
             "current source": ("white", "light gray"),
@@ -415,6 +433,238 @@ def get_palette(may_use_fancy_formats, theme="classic"):
         })
 
         # }}}
+    elif theme == "solarized":
+    # {{{ solarized
+        palette_dict.update({
+            # UI
+            "header": ("black", "light blue", "standout"),
+            "focused sidebar": ("yellow", "light blue", "standout"),
+            "group head": ("black", "light blue"),
+            "background": ("black", "light blue"),
+            "label": ("black", "light blue"),
+            "value": ("white", "dark blue"),
+            "fixed value": ("black", "light blue"),
+
+            "variables": ("light blue", "default"),
+
+            "var label": ("dark blue", "default"),
+            "var value": ("light blue", "default"),
+
+            "focused var label": ("white", "dark blue"),
+            "focused var value": ("black", "dark blue"),
+
+            "highlighted var label": ("white", "light green"),
+            "highlighted var value": ("white", "light green"),
+            "focused highlighted var label": ("white", "light green"),
+            "focused highlighted var value": ("white", "light green"),
+
+            "stack": ("light blue", "default"),
+
+            "frame name": ("dark blue", "default"),
+            "frame class": ("light blue", "default"),
+            "frame location": ("light green", "default"),
+
+            "focused frame name": ("white", "dark blue"),
+            "focused frame class": ("black", "dark blue"),
+            "focused frame location": ("dark gray", "dark blue"),
+
+            "focused current frame name": ("white", "light green"),
+            "focused current frame class": ("black", "light green"),
+            "focused current frame location": ("dark gray", "light green"),
+
+            "current frame name": ("white", "light green"),
+            "current frame class": ("black", "light green"),
+            "current frame location": ("dark gray", "light green"),
+
+            # breakpoints
+            "breakpoint": ("light blue", "default"),
+            "disabled breakpoint": ("light gray", "default"),
+            "focused breakpoint": ("white", "light green"),
+            "focused disabled breakpoint": ("light gray", "light green"),
+            "current breakpoint": ("white", "dark blue"),
+            "disabled current breakpoint": ("light gray", "dark blue"),
+            "focused current breakpoint": ("white", "light green"),
+            "focused disabled current breakpoint": ("light gray", "light green"),
+
+            # source
+            "breakpoint source": ("light blue", "black"),
+            "current breakpoint source": ("black", "light green"),
+            "breakpoint focused source": ("dark gray", "dark blue"),
+            "current breakpoint focused source": ("black", "light green"),
+            "breakpoint marker": ("dark red", "default"),
+
+            "search box": ("default", "default"),
+
+            "source": ("light blue", "default"),
+            "current source": ("light gray", "light blue"),
+            "current focused source": ("light gray", "light blue"),
+
+            "focused source": ("dark gray", "dark blue"),
+
+            "current highlighted source": ("black", "dark cyan"),
+            "highlighted source": ("light blue", "black"),
+
+            "line number": ("light blue", "default"),
+            "keyword": ("dark green", "default"),
+            "name": ("light blue", "default"),
+            "literal": ("dark cyan", "default"),
+            "string": ("dark cyan", "default"),
+            "doublestring": ("dark cyan", "default"),
+            "singlestring": ("light blue", "default"),
+            "docstring": ("dark cyan", "default"),
+            "backtick": ("light green", "default"),
+            "punctuation": ("light blue", "default"),
+            "comment": ("light green", "default"),
+            "classname": ("dark blue", "default"),
+            "funcname": ("dark blue", "default"),
+
+            # shell
+
+            "command line edit": ("light blue", "default"),
+            "command line prompt": ("light blue", "default"),
+
+            "command line output": ("light blue", "default"),
+            "command line input": ("light blue", "default"),
+            "command line error": ("dark red", "default"),
+
+            "focused command line output": ("black", "light green"),
+            "focused command line input": ("black", "light green"),
+            "focused command line error": ("dark red", "light blue"),
+
+            "command line clear button": ("light blue", "default"),
+            "command line focused button": ("black", "light blue"),
+        })
+
+    # }}}
+    elif theme == "agr-256":
+    # {{{ agr-256
+        palette_dict.update({
+            "header": ("h235", "h252", "standout"),
+
+            # {{{ variables view
+            "variables": ("h235", "h233"),
+            "variable separator": ("h23", "h252"),
+
+            "var label": ("h111", "h233"),
+            "var value": ("h255", "h233"),
+            "focused var label": ("h192", "h24"),
+            "focused var value": ("h192", "h24"),
+
+            "highlighted var label": ("h252", "h22"),
+            "highlighted var value": ("h255", "h22"),
+            "focused highlighted var label": ("h252", "h64"),
+            "focused highlighted var value": ("h255", "h64"),
+
+            "return label": ("h113", "h233"),
+            "return value": ("h113", "h233"),
+            "focused return label": (add_setting("h192", "bold"), "h24"),
+            "focused return value": ("h192", "h24"),
+            # }}}
+
+            # {{{ stack view
+            "stack": ("h235", "h233"),
+
+            "frame name": ("h192", "h233"),
+            "focused frame name": ("h192", "h24"),
+            "frame class": ("h111", "h233"),
+            "focused frame class": ("h192", "h24"),
+            "frame location": ("h252", "h233"),
+            "focused frame location": ("h192", "h24"),
+
+            "current frame name": ("h255", "h22"),
+            "focused current frame name": ("h255", "h64"),
+            "current frame class": ("h111", "h22"),
+            "focused current frame class": ("h255", "h64"),
+            "current frame location": ("h252", "h22"),
+            "focused current frame location": ("h255", "h64"),
+            # }}}
+
+            # {{{ breakpoint view
+            "breakpoint": ("h80", "h233"),
+            "disabled breakpoint": ("h60", "h233"),
+            "focused breakpoint": ("h192", "h24"),
+            "focused disabled breakpoint": ("h182", "h24"),
+            "current breakpoint": (add_setting("h255", "bold"), "h22"),
+            "disabled current breakpoint": (add_setting("h016", "bold"), "h22"),
+            "focused current breakpoint": (add_setting("h255", "bold"), "h64"),
+            "focused disabled current breakpoint": (add_setting("h016", "bold"), "h64"),
+            # }}}
+
+            # {{{ ui widgets
+
+            "selectable": ("h252", "h235"),
+            "focused selectable": ("h255", "h24"),
+
+            "button": ("h252", "h235"),
+            "focused button": ("h255", "h24"),
+
+            "background": ("h235", "h252"),
+            "hotkey": (add_setting("h235", "underline"), "h252", "underline"),
+            "focused sidebar": ("h23", "h252", "standout"),
+
+            "warning": (add_setting("h255", "bold"), "h124", "standout"),
+
+            "label": ("h235", "h252"),
+            "value": ("h255", "h17"),
+            "fixed value": ("h252", "h17"),
+            "group head": (add_setting("h25", "bold"), "h252"),
+
+            "search box": ("h255", "h235"),
+            "search not found": ("h255", "h124"),
+
+            "dialog title": (add_setting("h255", "bold"), "h235"),
+
+            # }}}
+
+            # {{{ source view
+            "breakpoint marker": ("h160", "h235"),
+
+            "breakpoint source": ("h252", "h124"),
+            "breakpoint focused source": ("h192", "h124"),
+            "current breakpoint source": ("h192", "h124"),
+            "current breakpoint focused source": (add_setting("h192", "bold"), "h124"),
+            # }}}
+
+            # {{{ highlighting
+            "source": ("h255", "h235"),
+            "focused source": ("h192", "h24"),
+            "highlighted source": ("h252", "h22"),
+            "current source": (add_setting("h252", "bold"), "h23"),
+            "current focused source": (add_setting("h192", "bold"), "h23"),
+            "current highlighted source": ("h255", "h22"),
+
+            "line number": ("h241", "h235"),
+            "keyword": ("h111", "h235"),
+
+            "literal": ("h173", "h235"),
+            "string": ("h113", "h235"),
+            "doublestring": ("h113", "h235"),
+            "singlestring": ("h113", "h235"),
+            "docstring": ("h113", "h235"),
+
+            "name": ("h192", "h235"),
+            "punctuation": ("h223", "h235"),
+            "comment": ("h246", "h235"),
+
+            # }}}
+
+            # {{{ shell
+            "command line edit": ("h255", "h233"),
+            "command line prompt": (add_setting("h192", "bold"), "h233"),
+
+            "command line output": ("h80", "h233"),
+            "command line input": ("h255", "h233"),
+            "command line error": ("h160", "h233"),
+
+            "focused command line output": (add_setting("h192", "bold"), "h24"),
+            "focused command line input": ("h255", "h24"),
+            "focused command line error": ("h235", "h24"),
+
+            "command line clear button": (add_setting("h255", "bold"), "h233"),
+            "command line focused button": ("h255", "h24"),
+            # }}}
+        })
+    # }}}
 
     else:
         try:
@@ -423,8 +673,8 @@ def get_palette(may_use_fancy_formats, theme="classic"):
                     "add_setting": add_setting,
                     }
 
-            from os.path import expanduser
-            execfile(expanduser(theme), symbols)
+            from os.path import expanduser, expandvars
+            execfile(expanduser(expandvars(theme)), symbols)
         except:
             print("Error when importing theme:")
             from traceback import print_exc
